@@ -5,7 +5,9 @@ config_file () {
     [ -f "$NEW" ] || { echo "$NEW does not exist!"; return; }
 
     if [ -f "$1" ] ; then
-	if cmp -s "$1" "$NEW" ; then
+	if [ "$2" = "-rm" ] ; then
+	    rm "$NEW"
+	elif cmp -s "$1" "$NEW" ; then
 	    rm "$NEW"
 	else
 	    echo
